import math, time
import cv2
import numpy as np

original = cv2.imread(input('Enter src image address:\n'))
contrast = cv2.imread(input("Enter encrypted image address:\n"))

def psnr(img1, img2):
    mse = np.mean((img1 - img2) ** 2)
    if mse == 0:
        return 100
    PIXEL_MAX = 255.0
    return 20 * math.log10(PIXEL_MAX / math.sqrt(mse))

d = psnr(original, contrast)
print(d)

time.sleep(10)



import sys


class Convert():

    def __init__(self):
        pass

    def convert_pw(self, password):
        self.pass_key =  sum([int(bin(ord(i))[2::]) for i in password])
        self.pass_key = sum([int(i) for i in str(self.pass_key)])
        return int(self.pass_key)

    def convert_text_helper(self, pass_key):
        self.key = int(int(pass_key)**0.5)
        if (int(self.key) > 99 or int(self.key) < 23):
            print('Shared Key error')
            sys.exit()
        else :
            print('Shared Key is valid')
            return self.key
        
    def secret_message(self, lst, key):
        self.list = lst
        self.key = key
        self.message = ''.join(self.list)
        temp = self.message.strip().split(',')
        temp_1 = [chr(int(i) + self.key) for i in temp if i.isdigit()]
        return ''.join(temp_1)



from .TextConverter import Convert
from .GetMatrix import ChangeMatrix
from .ImageReader import GetMatrix



class Encrypt():
    '''will import modules and do nessacary calculations and return to you an image with your message encrypted'''
    def __init__(self, text, password, address):
        
        #data is object and returns encoded data
        data = Convert()
        self.message = data.convert_ascii(text)     #a list of length(text)
        #print(self.message)
        self.pass_key = data.convert_sum(password)
        
        #returns 3D matrix of your given image address
        self.image = GetMatrix(address)
        #print(self.image.matrix)
        #print(self.message)

        x = 0
        #traversing through image matrix to do manipulations
        for i in range(0, len(self.image.matrix), self.pass_key):
            for j in range(0, len(self.image.matrix[i]), self.pass_key):
                if i == 0 and j == 0:
                    pass
                else:
                    if x < len(self.message):
                       # print(i, j)
                        #print(self.image.matrix[i][j], self.message[x])
                        self.image.matrix[i][j] = ChangeMatrix(self.image.matrix[i][j], self.message[x]).matrix_row
                        #print(self.image.matrix[i][j])
                        x += 1
                    else:
                        break
                    
        self.image.matrix[0][0][0] = self.pass_key
        self.image.matrix[0][0][1] = len(self.message)
        #reserved----- self.matrix[0][0][2]
        #print(self.pass_key)
        #print(self.image.matrix)

        #save matrix as .png
        self.verify = self.image.save(input('Enter the name of your encrypted image:\n'), self.image.matrix)  
                        
        #verify images
        #print(self.image.matrix)
        #print(ChangeMatrix(self.image.matrix).inverted_image)
        #print(self.verify)



class ChangeMatrix():

    def __init__(self, row, number):
        self.number = str(number)
        self.matrix_row = row
        #Checks the length of number if 0 returs since nothing to add
        '''
        if int(self.number) == 0:
               #has length eg. 1,2,3
            #print('length: ', self.length)
        else:
            return
        '''
        
        for i in range(1, len(self.matrix_row)):
            if len(self.number) == 1 and i == 2:
                self.matrix_row[-i] = str(round(int(self.matrix_row[-i]), -1))
                break
            self.matrix_row[-i] = str(round(int(self.matrix_row[-i]), -1) + int(self.number[-i]))
        #print(self.matrix_row)



import cv2 as cv

class GetMatrix():

    def __init__(self, address):

        self.matrix = cv.imread(address)
        #self.matrix = cv.resize(self.image, (0, 0), fx = 0.5, fy = 0.5)
        return

    def save(self, filename, matrix):
        self.filename = filename
        cv.imwrite(filename, matrix)
        #(Image.fromarray(matrix)).save(filename)
        print('Image saved successfully.')
        self.verify = cv.imread(filename)
        return self.verify




from .Decrypt import Decrypt
import time



print(f'''If you encounter any 'Error decrypting' or is your "Shared Key" is less than {22**2} or greater than {99**2} you might get a wrong message.
In such case ask the sender to replace white spaces with underscores or ask them not to use any symbols''')


Decrypt(input('\nEnter image address:\n'), input('Enter your password:\n'), input('Enter your security key:\n'))
time.sleep(5)

#Decrypt('Decrypt\\finally_done.png', '3020128', '1024')#



from .GetMatrix import GetMatrix
from .TextConverter import Convert
import sys

class Decrypt():

    def __init__(self, address, password, shared_key):

        self.matrix = GetMatrix(address)
        if not GetMatrix(address):
            print('address error')
            sys.exit()
        data = Convert()
        self.pass_key = data.convert_pw(password)

        #Check if password is correct
        if self.pass_key != int(self.matrix.matrix[0][0][0]):
            print('Incorrect Password !')
            sys.exit()
        else:
            print('Passwords match...')
        self.shared_key =  data.convert_text_helper(shared_key)

        #extract list of characters and add pass key to decrypt the text
        self.extracted_text = []
        self.length = int(self.matrix.matrix[0][0][1])
        x = 0
        for i in range(0, len(self.matrix.matrix), self.pass_key):
            for m in range(0, len(self.matrix.matrix[i]), self.pass_key):
                if i == 0 and m == 0:
                    pass
                elif x < self.length:
                    #print(i, m)
                    for k in range(1, len(self.matrix.matrix[i][m])):
                        self.extracted_text.append(str(self.matrix.matrix[i][m][k])[-1])
                        if  len(str(self.matrix.matrix[i][m][-1])) > 1:
                            if int(str(self.matrix.matrix[i][m][k])[-k]) == 0:
                                break
                    self.extracted_text.append(',')
                    x += 1
                else:
                    break
        self.message = data.secret_message(self.extracted_text, self.shared_key)
        print('\n\n', self.message)

        




import sys

class Convert():

    def __init__(self):
        pass

   
    def convert_sum(self, password):
        self.pass_key =  sum([int(bin(ord(i))[2::]) for i in password])
        self.pass_key = sum([int(i) for i in str(self.pass_key)])
        #print(self.pass_key)
        return self.pass_key

    
    def convert_ascii(self, text):
        secret_message = [ord(i) for i in text]
        self.private_key = min(secret_message)
        self.message = [int(i-self.private_key) for i in secret_message]
        if self.private_key < 23 or max(self.message) > 99:
            print('Error encrypting')
            sys.exit()
        print('Your private key is: ',(self.private_key)**2)
        #print(self.message)
        return self.message



from .Encrypt import Encrypt


print('''Thankyou for using our service.\n1.) We request you to not use any symbols as they are out of our range.
2.) If you get 'Error Encrypting' then replace all symbols if used or replace white spaces with underscores ('_').\n\n''')


Encrypt((input("\nEnter your message:\n").strip()), (input('\nEnter your password:\n').strip()),
        (input('\nEnter image address:\n ').strip()))

#Encrypt('My name is Justin', '3020128', 'car.jpg')
